require('shelljs/global') // Shelljs就是基于node的一层命令封装插件，让前端开发者可以不依赖linux也不依赖类似于cmder的转换工具，而是直接在我们最熟悉不过的javascript代码中编写shell命令实现功能。
const { prompt } = require('inquirer') //控制台交互
const download = require('download-git-repo') //下载并提取 git 仓库。
const handlebars = require('handlebars')  //package.json 渲染模版

const chalk = require('chalk') //控制台字体颜色
const ora = require('ora') //控制台动画、视觉
const symbols = require('log-symbols'); //控制台对错图标

const fs = require('fs')
const path = require('path')


module.exports = function (options) {
    const {Option}=options
    const question = [
        {
            type: 'input',
            name: 'name',
            message: '项目名称',
            default: 'davidbean-project',
            filter (val) {
                return val.trim()
            },
            validate (val) {
                const validate = val.trim().length > 0
                if (!validate) {
                    return '项目名称不能为空'
                } else if (fs.existsSync(`./${val}`)) {
                    return `当前目录已存在项目：${val}`
                }
                return true
            },
            transformer (val) {
                return val
            }
        },
        {
            type:"list",
            message:"请选择下载分支：",
            name:"branch",
            default:"master",
            choices:[
                "master",
                "dev",
            ],
        },
    ]
    prompt(question).then(({name, branch}) => {
        const gitPlace = require('../package.json').init.path
        const spinner = ora('模板下载中...')
        spinner.start()
        download(`https://github.com:${gitPlace}#${branch}`, `./${name}`, {clone: true}, (err) => {
            if (err) {
                spinner.fail();
                console.log(symbols.error, chalk.red(err));
                removeDir(`./${name}`)
                process.exit()
            }
            spinner.succeed();
            // 删除不需要的文件
            // removeDir(`./${name}`)
            console.log(symbols.success, `${chalk.green('David 的Vue模版下载好啦，搞起来哥们')}`)
            // 修改文件
            // const packageJson = JSON.parse(fs.readFileSync(`./${name}/package.json`).toString())
            // packageJson.name = name
            // packageJson.description = description
            // packageJson.author = author
            // fs.writeFileSync(`./${name}/package.json`, JSON.stringify(packageJson, null, 2))
            // console.log(chalk.green('模板下载完成，将自动安装依赖...'))


            // eslint-disable-next-line no-undef
            //执行命令
            // exec(`cd ${name} npm install`)
            //
            // console.log(`${chalk.green('项目安装完成，请尽情使用吧')}`)
            // console.log(`${chalk.yellow${name}`)}${chalk.yellow('npm run dev')}`)
        })
    })
}

function removeDir(p, del) {
    const statObj = fs.statSync(p) // fs.statSync同步读取文件状态，判断是文件目录还是文件。
    if (statObj.isDirectory()) { // 如果是目录
        const dirs = fs.readdirSync(p) // fs.readdirSync()同步的读取目标下的文件 返回一个不包括 '.' 和 '..' 的文件名的数组['b','a']
        dirs.forEach(dir => {
            if (del) {
              removeDir(path.join(p, dir), true)
            }
        })
        del && fs.rmdirSync(p) // 删除目录
    } else {
        fs.unlinkSync(p) // 删除文件
    }
}
function copyDir(src, dst) {
    const statObj = fs.statSync(src) // fs.statSync同步读取文件状态，判断是文件目录还是文件。
    if (statObj.isDirectory()) { // 如果是目录
        !fs.existsSync(dst) && fs.mkdirSync(dst)
        const dirs = fs.readdirSync(src) // fs.readdirSync()同步的读取目标下的文件 返回一个不包括 '.' 和 '..' 的文件名的数组['b','a']
        dirs.forEach(dir => {
            copyDir(path.join(src, dir), path.join(dst, dir))
        })
    } else {
        // 改为同步
        // const readable = fs.createReadStream(src)// 创建读取流
        // const writable = fs.createWriteStream(dst)// 创建写入流
        // readable.pipe(writable)
        fs.writeFileSync(dst, fs.readFileSync(src))
    }
}
